# -*- coding: utf-8 -*-
import configparser
import os
import re

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.management.utils import get_random_secret_key

from django_settings_custom import encryption


class Command(BaseCommand):
    """
    A Django interactive command for configuration file generation.
    """
    help = 'A Django interactive command for configuration file generation.'
    usage = 'python manage.py generate_settings path/to/template/settings.ini target/path/of/settings.ini'

    settings_template_file = settings.SETTINGS_TEMPLATE_FILE
    settings_file_path = settings.SETTINGS_FILE_PATH

    def add_arguments(self, parser):
        parser.usage = self.usage
        parser.add_argument(
            'settings_template_file',
            nargs='?',
            type=str,
            default=None,
            help="Path to the settings template file.")
        parser.add_argument(
            'settings_file_path',
            nargs='?',
            type=str,
            default=None,
            help="Target path for the settings file.")

    @staticmethod
    def get_value(section, key, value_type, secret_key):
        """
        Get a value for the [section] key passed as parameter.
        :param section: Section in the configuration file.
        :param key: Key in the configuration file.
        :param value_type: Value type read in template.
        :param secret_key: Django secret key
        :return: Value for the [section] key
        """
        value = None
        if value_type == 'DJANGO_SECRET_KEY':
            value = secret_key
        elif 'USER_VALUE' in value_type:
            encrypt = value_type == 'ENCRYPTED_USER_VALUE'
            encrypt_message = ''
            if encrypt:
                encrypt_message = '(will be encrypted) '
            value = input('Value for [%s] %s %s: ' % (section, key, encrypt_message))
            if encrypt:
                value = encryption.encrypt(value, secret_key)
        return value

    def handle(self, *args, **options):
        settings_template_file = options['settings_template_file']
        settings_file_path = options['settings_file_path']
        if not settings_template_file and not settings_file_path:
            settings_template_file = self.settings_template_file
            settings_file_path = self.settings_file_path
        if not settings_template_file:
            raise CommandError('Parameter settings_template_file undefined.\nUsage: %s' % self.usage)
        if not settings_file_path:
            raise CommandError('Parameter settings_file_path undefined.\nUsage: %s' % self.usage)
        if not os.path.exists(settings_template_file):
            raise CommandError('The settings template file doesn\'t exists.')

        self.stdout.write('** Configuration file generation: **')
        if os.path.exists(settings_file_path):
            override = input('A configuration file already exists at %s. Would you override it ? (y/N) : ' %
                             settings_file_path)
            if override.upper() != 'Y':
                raise CommandError('Generation cancelled.')

        config = configparser.ConfigParser()
        config.optionxform = str
        config.read(settings_template_file)

        generate_secret_key = input('Do you want to generate the secret key for Django ? (Y/n) : ')
        if generate_secret_key.upper() == 'N':
            secret_key = input('Enter your secret key : ')
            if not secret_key:
                raise CommandError('Django secret key is needed for encryption. Generation cancelled.')
        else:
            secret_key = get_random_secret_key()
            self.stdout.write('Django secret key generated !')
        secret_key = secret_key.replace('%', '%%')

        self.stdout.write('\n** Enter values for configuration file content **')
        variable_regex = re.compile(r' *{(.+)} *')
        for section, values in config.items():
            for key, value in values.items():
                match_groups = variable_regex.match(value)
                if match_groups:
                    value_type = match_groups.group(1).strip().upper()
                    config.set(section, key, self.get_value(section, key, value_type, secret_key))
        self.stdout.write('\nWriting file at %s:' % settings_file_path)
        os.makedirs(os.path.dirname(settings_file_path), exist_ok=True)
        with open(settings_file_path, 'w') as config_file:
            config.write(config_file)
        self.stdout.write(self.style.SUCCESS('Configuration file successfully generated !'))



# -*- coding: utf-8 -*-
import base64

from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random

from django.conf import settings


def _compute_key(secret_key=None):
    """
    Compute a valid key for AES crypto algorithm.
    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    :param secret_key: The key for encryption, or None if you want use the SECRET_KEY.
    :return: A valid key for AES.
    """
    if secret_key is None:
        secret_key = settings.SECRET_KEY
    return SHA256.new(bytes(secret_key, 'utf-8')).digest()


def encrypt(source, secret_key=None):
    """
    Encrypt the source with the key passed as parameter.
    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    :param source: A string or a bytes array to encrypt.
    :param secret_key: The key for encryption, or None if you want use the SECRET_KEY.
    :return: Encrypted value.
    """
    if isinstance(source, str):
        source = bytes(source, 'utf-8')
    key = _compute_key(secret_key)
    iv = Random.new().read(AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    padding = AES.block_size - len(source) % AES.block_size
    source += bytes([padding]) * padding
    data = iv + cipher.encrypt(source)
    return base64.b64encode(data).decode('latin-1')


def decrypt(source, secret_key=None):
    """
    Decrypt the source with the key passed as parameter.
    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    :param source: A string or a bytes array to decrypt.
    :param secret_key: The key for encryption, or None if you want use the SECRET_KEY.
    :return: Decrypted value.
    """
    key = _compute_key(secret_key)
    source = base64.b64decode(source.encode('latin-1'))
    iv = source[:AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    data = cipher.decrypt(source[AES.block_size:])
    padding = data[-1]
    if data[-padding:] != bytes([padding]) * padding:
        raise ValueError('Error in decryption.')
    return data[:-padding].decode("utf-8")




# -*- coding: utf-8 -*-
import os

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements
from setuptools import find_packages, setup

requirements = [str(ir.req) for ir in parse_requirements('requirements.txt', session=False)]
requirements_test = [r for r in [str(ir.req) for ir in parse_requirements('requirements-dev.txt', session=False)]
                     if r not in requirements]

long_description = """Provide a Django interactive command for configuration file generation.
  See the project page for more information:
  https://github.com/ThomasMarques/django-settings-custom"""
if os.path.isfile("README.md"):
    with open("README.md") as f:
        long_description = f.read()


def get_version_info():
    return '1.0.4'


setup(
    name='django-settings-custom',
    version=get_version_info(),
    description="A Django interactive command for configuration file generation.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Thomas Marques',
    author_email='marquesthom@gmail.com',
    license='MIT License',
    platforms=['any'],
    url='https://github.com/ThomasMarques/django-settings-custom',
    packages=find_packages(where='.',),
    include_package_data=True,
    install_requires=requirements,
    extras_require={'test': requirements_test},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ]
)
